@@ -64,7 +64,6 @@ const kNumFolderViews = 4; // total number of folder views
 var gMessagePane;
 
 var gThreadAndMessagePaneSplitter = null;
-var gStartFolderUri = null;
 /**
  * Tracks whether the right mouse button changed the selection or not.  If the
  * user right clicks on the selection, it stays the same.  If they click outside
@@ -322,19 +321,34 @@ function LoadPostAccountWizard()
 
   //need to add to session before trying to load start folder otherwise listeners aren't
   //set up correctly.
-  // argument[0] --> folder uri
-  if ("arguments" in window)
+
+  let startFolderURI = null, startMsgHdr = null;
+  if ("arguments" in window && window.arguments.length > 0)
   {
-    // filter our any feed urls that came in as arguments to the new window...
-    if (window.arguments.length && /^feed:/i.test(window.arguments[0] ))
+    let arg0 = window.arguments[0];
+    // If the argument is a string, it is either a folder URI or a feed URI
+    if (typeof arg0 == "string")
     {
-      var feedHandler = Components.classes["@mozilla.org/newsblog-feed-downloader;1"].getService(Components.interfaces.nsINewsBlogFeedDownloader);
-      if (feedHandler)
-        feedHandler.subscribeToFeed(window.arguments[0], null, msgWindow);
-      gStartFolderUri = null;
+      // filter our any feed urls that came in as arguments to the new window...
+      if (/^feed:/i.test(arg0))
+      {
+        let feedHandler = Components.classes["@mozilla.org/newsblog-feed-downloader;1"]
+          .getService(Components.interfaces.nsINewsBlogFeedDownloader);
+        if (feedHandler)
+          feedHandler.subscribeToFeed(arg0, null, msgWindow);
+      }
+      else
+      {
+        startFolderURI = arg0;
+      }
+    }
+    else if (arg0)
+    {
+      // arg0 is an object
+      if (("wrappedJSObject" in arg0) && arg0.wrappedJSObject)
+        arg0 = arg0.wrappedJSObject;
+      startMsgHdr = ("msgHdr" in arg0) ? arg0.msgHdr : null;
     }
-    else
-      gStartFolderUri = (window.arguments.length > 0) ? window.arguments[0] : null;
   }
 
   function completeStartup() {
@@ -394,7 +408,10 @@ function LoadPostAccountWizard()
   //  gFolderDisplay.ensureRowIsVisible use settimeout itself to defer that
   //  calculation, but that was ugly.  Also, in theory we will open the window
   //  faster if we let the event loop start doing things sooner.
-  window.setTimeout(loadStartFolder, 0, gStartFolderUri);
+  if (startMsgHdr)
+    window.setTimeout(loadStartMsgHdr, 0, startMsgHdr);
+  else
+    window.setTimeout(loadStartFolder, 0, startFolderURI);
 }
 
 function HandleAppCommandEvent(evt)
@@ -423,6 +440,25 @@ function HandleAppCommandEvent(evt)
   }
 }
 
+/**
+ * Look for another 3-pane window.
+ */
+function FindOther3PaneWindow()
+{
+  let windowMediator =
+    Components.classes["@mozilla.org/appshell/window-mediator;1"]
+        .getService(Components.interfaces.nsIWindowMediator);
+  // XXX We'd like to use getZOrderDOMWindowEnumerator here, but it doesn't work
+  // on Linux
+  let enumerator = windowMediator.getEnumerator("mail:3pane");
+  while (enumerator.hasMoreElements()) {
+    let win = enumerator.getNext();
+    if (win != window)
+      return win;
+  }
+  return null;
+}
+
 /**
  * Called by messenger.xul:onunload, the 3-pane window inside of tabs window.
  *  It's being unloaded!  Right now!
@@ -433,8 +469,34 @@ function OnUnloadMessenger()
   gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
   gPrefBranch.removeObserver("mail.pane_config.dynamic", MailPrefObserver);
 
-  // - Persist the tab state and then close the tabs.
+  // - Persist the tab state and then close the tabs. If another 3-pane window
+  //   is open, do not persist tab state.
   // XXX do not assume there is only ever one 3-pane.
+  if (!FindOther3PaneWindow())
+    persistTabState();
+
+  let tabmail = document.getElementById("tabmail");
+  tabmail.closeTabs();
+
+  var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
+                              .getService(Components.interfaces.nsIMsgMailSession);
+  mailSession.RemoveFolderListener(folderListener);
+
+  gPhishingDetector.shutdown();
+
+  // FIX ME - later we will be able to use onload from the overlay
+  OnUnloadMsgHeaderPane();
+
+  UnloadPanes();
+
+  OnMailWindowUnload();
+}
+
+/**
+ * This currently only happens at window unload.
+ */
+function persistTabState()
+{
   let tabmail = document.getElementById('tabmail');
   let tabsState = tabmail.persistTabs();
   // build the state like we aren't assuming a single 3-pane
@@ -456,22 +518,6 @@ function OnUnloadMessenger()
   foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
   foStream.write(data, data.length);
   foStream.close();
-
-  tabmail.closeTabs();
-
-
-  var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
-                              .getService(Components.interfaces.nsIMsgMailSession);
-  mailSession.RemoveFolderListener(folderListener);
-
-  gPhishingDetector.shutdown();
-
-  // FIX ME - later we will be able to use onload from the overlay
-  OnUnloadMsgHeaderPane();
-
-  UnloadPanes();
-
-  OnMailWindowUnload();
 }
 
 // XXX provides GlodaUtils, remove once we migrate loadFileToString
@@ -479,9 +525,14 @@ Components.utils.import("resource://app/modules/gloda/utils.js");
 
 /**
  * Attempt to restore our tab states.  This should only be called by
- *  loadStartFolder.
+ * |loadStartFolder| or |loadStartMsgHdr|.
+ *
+ * @param aDontRestoreFirstTab If this is true, the first tab will not be
+ *                             restored, and will continue to retain focus at
+ *                             the end. This is needed if the window was opened
+ *                             with a folder or a message as an argument.
  */
-function atStartupRestoreTabs() {
+function atStartupRestoreTabs(aDontRestoreFirstTab) {
   let file = Components.classes["@mozilla.org/file/directory_service;1"]
                        .getService(Components.interfaces.nsIProperties)
                        .get("ProfD", Components.interfaces.nsIFile);
@@ -501,7 +552,7 @@ function atStartupRestoreTabs() {
   let state = JSON.parse(data);
   let tabsState = state.windows[0].tabs;
   let tabmail = document.getElementById('tabmail');
-  tabmail.restoreTabs(tabsState);
+  tabmail.restoreTabs(tabsState, aDontRestoreFirstTab);
 
   return true;
 }
@@ -515,6 +566,22 @@ function loadExtraTabs()
   }
 }
 
+/**
+ * Loads the given message header at window open. Exactly one out of this and
+ * |loadStartFolder| should be called.
+ *
+ * @param aStartMsgHdr The message header to load at window open
+ */
+function loadStartMsgHdr(aStartMsgHdr)
+{
+  setTimeout(loadExtraTabs, 0);
+
+  // We'll just clobber the default tab
+  atStartupRestoreTabs(true);
+
+  MsgDisplayMessageInFolderTab(aStartMsgHdr);
+}
+
 function loadStartFolder(initialUri)
 {
   setTimeout(loadExtraTabs, 0);
@@ -523,27 +590,26 @@ function loadStartFolder(initialUri)
     var isLoginAtStartUpEnabled = false;
 
     if (!initialUri) {
-      // Look to see if a master password is set, if so prompt for it to try
-      // and avoid the multiple master password prompts on startup scenario.
+      // Try to avoid the multiple master password prompts on startup scenario
+      // by prompting for the master password upfront.
       let token =
         Components.classes["@mozilla.org/security/pk11tokendb;1"]
                   .getService(Components.interfaces.nsIPK11TokenDB)
                   .getInternalKeyToken();
 
-      // If an empty string is valid for the internal token, then we don't
-      // have a master password, else, if it does, then try to login.
+      // Only log in to the internal token if it is already initialized,
+      // otherwise we get a "Change Master Password" dialog.
       try {
-        if (!token.checkPassword(""))
+        if (!token.needsUserInit)
           token.login(false);
       }
       catch (ex) {
-      // If user cancels an exception is expected. checkPassword also
-      // seems to fail w/ mozmill.
+      // If user cancels an exception is expected.
       }
     }
 
-    let loadFolder = !atStartupRestoreTabs();
     // If a URI was explicitly specified, we'll just clobber the default tab
+    let loadFolder = !atStartupRestoreTabs(!!initialUri);
     if (initialUri)
       loadFolder = true;
 
@@ -713,7 +779,12 @@ function UpgradeProfileAndBeUglyAboutIt()
 
       for each (let server in fixIterator(servers, Components.interfaces.nsIMsgIncomingServer))
       {
-        if (server.type != "imap")
+        // If it's not an imap server, or if we haven't set the
+        // offlineDownload flag on the server, then don't set the offline
+        // flags on all the sub-folders.
+        // Note: We need to use instanceof to get the offlineDownload flag.
+        if (!(server instanceof Components.interfaces.nsIImapIncomingServer) ||
+            !server.offlineDownload)
           continue;
 
         let allFolders = Components.classes["@mozilla.org/supports-array;1"]
@@ -943,6 +1014,8 @@ function FolderPaneOnClick(event)
 
 function OpenMessageInNewTab(event)
 {
+  if (!gFolderDisplay.selectedMessage)
+    return;
   var bgLoad = gPrefBranch.getBoolPref("mail.tabs.loadInBackground");
   if (event.shiftKey)
     bgLoad = !bgLoad;
@@ -1089,17 +1162,65 @@ function ThreadPaneOnDragStart(aEvent) {
   if (aEvent.originalTarget.localName != "treechildren")
     return;
 
-  var messages = gFolderDisplay.selectedMessageUris;
+  let messages = gFolderDisplay.selectedMessageUris;
   if (!messages)
     return;
 
   gFolderDisplay.hintAboutToDeleteMessages();
-  for (let i in messages)
+  let ios = Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService);
+  let fileNames = [];
+  let msgUrls = {};
+
+  // dragging multiple messages to desktop does not
+  // currently work, pending core fixes for
+  // multiple-drop-on-desktop support. (bug 513464)
+  for (let i in messages) {
+    messenger.messageServiceFromURI(messages[i])
+             .GetUrlForUri(messages[i], msgUrls, null);
+    var subject = messenger.messageServiceFromURI(messages[i])
+                           .messageURIToMsgHdr(messages[i]).mime2DecodedSubject;
+    var uniqueFileName = suggestUniqueFileName(subject.substr(0,124), ".eml",
+                                               fileNames);
+    fileNames[i] = uniqueFileName;
     aEvent.dataTransfer.mozSetDataAt("text/x-moz-message", messages[i], i);
+    aEvent.dataTransfer.mozSetDataAt("text/x-moz-url",msgUrls.value.spec, i);
+    aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise-url",
+                                     msgUrls.value.spec + "?fileName=" + uniqueFileName,
+                                     i);
+    aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise", null, i);
+  }
   aEvent.dataTransfer.effectAllowed = "copyMove";
   aEvent.dataTransfer.addElement(aEvent.originalTarget);
 }
 
+/**
+ * example use:
+ *   suggestUniqueFileName("testname",".txt",["testname", "testname1"])
+ *   returns "testname2"
+ * does not check file system for existing files
+ * @param existingNames array of names in use
+ */
+function suggestUniqueFileName(identifier, type, existingNames) {
+  let suffix = 1;
+  let suggestion;
+  let base = identifier;
+  let exists;
+  do {
+    exists = false;
+    suggestion = GenerateValidFilename(base, type);
+    for (let i = 0; i < existingNames.length; i++) {
+      if (existingNames[i] == suggestion) {
+        base = identifier + suffix;
+        suffix++;
+        exists = true;
+        break;
+      }
+    }
+  } while (exists);
+  return suggestion;
+}
+
 function ThreadPaneOnDragOver(aEvent) {
   let ds = Components.classes["@mozilla.org/widget/dragservice;1"]
                      .getService(Components.interfaces.nsIDragService)
